@@ -6,6 +6,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <signal.h>
 #include <sys/types.h>
 
 #ifdef HAVE_DIRENT_H
@@ -62,7 +63,7 @@ extern Tcl_Interp *xcinterp;
 #endif
 
 extern Display    *dpy;
-extern Clientdata areastruct;
+extern XCWindowData *areawin;
 extern ApplicationData appdata;
 extern int 	  *appcolors;
 extern short	  popups;     /* total number of popup windows */
@@ -105,7 +106,7 @@ void dragfilebox(xcWidget w, caddr_t clientdata, XMotionEvent *event)
    if (filenum == flcurrent) return;
 
    if (flcurrent >= 0) 		/* erase previous box */
-      XDrawRectangle(dpy, lwin, areastruct.gc, 5,
+      XDrawRectangle(dpy, lwin, areawin->gc, 5,
 	   10 + FILECHARHEIGHT * (flcurrent
 	   - flstart), flcurwidth + 10, FILECHARHEIGHT);
 
@@ -113,7 +114,7 @@ void dragfilebox(xcWidget w, caddr_t clientdata, XMotionEvent *event)
 
    twidth = XTextWidth(appdata.filefont, files[filenum].filename,
 	    strlen(files[filenum].filename));
-   XDrawRectangle(dpy, lwin, areastruct.gc, 5,
+   XDrawRectangle(dpy, lwin, areawin->gc, 5,
 	   10 + FILECHARHEIGHT * (filenum
 	   - flstart), twidth + 10, FILECHARHEIGHT);
 
@@ -134,17 +135,17 @@ void startfiletrack(xcWidget w, caddr_t clientdata, XCrossingEvent *event)
    xcAddEventHandler(w, PointerMotionMask, False, (xcEventHandler)dragfilebox, NULL);
 #endif
 
-   XSetFunction(dpy, areastruct.gc, GXcopy);
-   XSetForeground(dpy, areastruct.gc, AUXCOLOR);
-   XSetLineAttributes(dpy, areastruct.gc, 1, LineSolid, CapRound, JoinMiter);
+   XSetFunction(dpy, areawin->gc, GXcopy);
+   XSetForeground(dpy, areawin->gc, AUXCOLOR);
+   XSetLineAttributes(dpy, areawin->gc, 1, LineSolid, CapRound, JoinMiter);
 
    /* draw initial box */
 
    flcurrent = -1;
    dragfilebox(w, NULL, (XMotionEvent *)event);
 
-   XSetFunction(dpy, areastruct.gc, GXxor);
-   XSetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
+   XSetFunction(dpy, areawin->gc, GXxor);
+   XSetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
 }
 
 /*-------------------------------------------------------------------------*/
@@ -155,7 +156,7 @@ void endfiletrack(xcWidget w, caddr_t clientdata, XCrossingEvent *event)
 {
    Window lwin = xcWindow(w);
 
-   XDrawRectangle(dpy, lwin, areastruct.gc, 5,
+   XDrawRectangle(dpy, lwin, areawin->gc, 5,
 	   10 + FILECHARHEIGHT * (flcurrent
 	   - flstart), flcurwidth + 10, FILECHARHEIGHT);
 
@@ -168,8 +169,8 @@ void endfiletrack(xcWidget w, caddr_t clientdata, XCrossingEvent *event)
 #endif
 
    /* Restore graphics state values */
-   XSetForeground(dpy, areastruct.gc, areastruct.gccolor);
-   XSetFunction(dpy, areastruct.gc, areastruct.gctype);
+   XSetForeground(dpy, areawin->gc, areawin->gccolor);
+   XSetFunction(dpy, areawin->gc, areawin->gctype);
 }
 
 /*----------------------------------------------------------------------*/
@@ -243,8 +244,25 @@ void findcrashfiles()
       sprintf(_STR, "%s/%s", xobjs.tempdir, dp->d_name);
       snptr = _STR + strlen(xobjs.tempdir) + 1;
       if (!strncmp(snptr, "XC", 2)) {
+	 char *dotptr = strchr(snptr, '.');
+	 pid = -1;
+	 if (dotptr && dotptr > snptr + 2) {
+	    *dotptr = '\0';
+	    if (sscanf(snptr + 2, "%d", &pid) != 1) 
+	       pid = -1;
+	    *dotptr = '.';
+         }
          if ((!stat(_STR, &sbuf)) && (sbuf.st_uid == userid)) {
 	    if ((recent == 0) || (sbuf.st_ctime > recent)) {
+
+	       /* Check if the PID belongs to an active process	*/
+	       /* by sending a CONT signal and checking if 	*/
+	       /* there was no error result.			*/
+
+	       if (pid != -1)
+		  if (kill((pid_t)pid, SIGCONT) == 0)
+		     continue;
+
 	       recent = sbuf.st_ctime;
 	       strcpy(_STR2, _STR);
 	    }
@@ -376,7 +394,7 @@ void listfiles(xcWidget w, popupstruct *okaystruct, caddr_t calldata)
 	    else if (match_filter(dp->d_name, filter))
 	       files[flfiles].filetype = MATCH;
 	    else {
-	       if (areastruct.filefilter)
+	       if (xobjs.filefilter)
 		  continue;
 	       else
 	          files[flfiles].filetype = NONMATCH;
@@ -405,7 +423,7 @@ void listfiles(xcWidget w, popupstruct *okaystruct, caddr_t calldata)
       pixheight = flfiles * FILECHARHEIGHT + 25;
       if (pixheight < textheight) pixheight = textheight;
 
-      flistpix = XCreatePixmap(dpy, areastruct.areawin, textwidth, pixheight,
+      flistpix = XCreatePixmap(dpy, areawin->window, textwidth, pixheight,
 	   DefaultDepthOfScreen(xcScreen(w)));
 
       /* Write the filenames onto the pixmap */
@@ -461,7 +479,7 @@ void newfilelist(xcWidget w, popupstruct *okaystruct)
       Wprintf("Error: Bad variable $XCOps(filter) in Tcl!");
       return;  
    }
-   areastruct.filefilter = bval;
+   xobjs.filefilter = bval;
 #endif
 
    for (n = 0; n < flfiles; n++)
@@ -686,7 +704,7 @@ void draglscroll(xcWidget w, popupstruct *okaystruct, XButtonEvent *event)
 
 void setfilefilter(xcWidget w, popupstruct *okaystruct, caddr_t calldata)
 {
-   areastruct.filefilter = (areastruct.filefilter) ? False : True;
+   xobjs.filefilter = (xobjs.filefilter) ? False : True;
 
    /* Force regeneration of the file list */
 
@@ -776,7 +794,7 @@ void genfilelist(xcWidget parent, popupstruct *okaystruct, Dimension width)
    wwidth = XTextWidth(appdata.xcfont, "filter", strlen("filter"));
    XtnSetArg(XtNx, width - wwidth - 50);
    XtnSetArg(XtNy, LISTHEIGHT + 10);
-   XtnSetArg(XtNset, areastruct.filefilter);
+   XtnSetArg(XtNset, xobjs.filefilter);
    XtnSetArg(XtNsquare, True);
    XtnSetArg(XtNborderWidth, 0);
    XtnSetArg(XtNfont, appdata.xcfont);
