@@ -40,7 +40,7 @@
 
 extern char _STR[150];
 extern Globaldata xobjs;
-extern Clientdata areastruct;
+extern XCWindowData *areawin;
 extern Display *dpy;
 extern int *appcolors;
 
@@ -124,14 +124,14 @@ void loadlgf(int mode)
 
    if (mode == 0) {
       reset(topobject, NORMAL);
-      pagereset(areastruct.page); 
+      pagereset(areawin->page); 
    }
 
    /* Set up filename and object (page) name */
 
-   xobjs.pagelist[areastruct.page]->filename = (char *) realloc (
-      xobjs.pagelist[areastruct.page]->filename, (strlen(inname) + 1) * sizeof(char));
-   strcpy(xobjs.pagelist[areastruct.page]->filename, inname);
+   xobjs.pagelist[areawin->page]->filename = (char *) realloc (
+      xobjs.pagelist[areawin->page]->filename, (strlen(inname) + 1) * sizeof(char));
+   strcpy(xobjs.pagelist[areawin->page]->filename, inname);
 
    /* If the filename has a path component, use only the root */
 
@@ -140,7 +140,7 @@ void loadlgf(int mode)
    else
       sprintf(topobject->name, "%s", inname);
 
-   renamepage(areastruct.page);
+   renamepage(areawin->page);
    printname(topobject);
 
    /* Read objects */
@@ -423,10 +423,8 @@ void loadlgf(int mode)
 			   labeldefaults(*newlabel, False, (*newinst)->position.x,
 				(*newinst)->position.y);
 			   (*newlabel)->color = DEFAULTCOLOR;
-		  	   if (areastruct.schemon){
-			      (*newlabel)->pin = LOCAL;
-			      (*newlabel)->color = LOCALPINCOLOR;
-		  	   }
+			   (*newlabel)->pin = LOCAL;
+			   (*newlabel)->color = LOCALPINCOLOR;
 			   if (!strcmp(tmpstring, "TO"))
 			      (*newlabel)->position.x += ((flip) ? 48 : -48);
 			   else
@@ -515,8 +513,8 @@ void loadlgf(int mode)
 	 }
       }
 
-   calcbbox(areastruct.topinstance);
-   centerview(areastruct.topinstance);
+   calcbbox(areawin->topinstance);
+   centerview(areawin->topinstance);
 
    for (i = 0; i < sigs; i++) free(signals[i]);
    free(signals);
@@ -567,7 +565,7 @@ void loadmat4(caddr_t nullval)
    if ((pdchar = strstr(_STR, ".ps")) != NULL) *pdchar = '\0';
    sprintf(topobject->name, "%s", _STR);
    Wprintf("Loaded file: %s", inname);
-   renamepage(areastruct.page);
+   renamepage(areawin->page);
    printname(topobject);
 
    /* Create input string buffer */
