@@ -718,6 +718,9 @@ void default_keybindings()
 /* add_binding(ALL_WINDOWS, ALT | XK_f, XCF_SPICEflat);	*/
 /* add_binding(ALL_WINDOWS, ALT | XK_p, XCF_PCB);		*/
 
+   /* Avoid spurious Num_Lock messages */
+   add_binding(ALL_WINDOWS, XK_Num_Lock, XCF_Nothing);
+
    /* 2-button vs. 3-button mouse bindings (set with -2	*/
    /* commandline option; 3-button bindings default)	*/
 
