@@ -37,18 +37,36 @@ proc xcircuit::getsubckttext {name} {
 }
 
 #---------------------------------------------------------------------------
-# Create a matching symbol for a subcircuit page.
-# This is a replacement for the procedure promptmakesymbol in wrapper.tcl.
+# Create a matching symbol for a subcircuit page or SPICE file.
+#
+# This is a replacement for the procedure makesymbol in wrapper.tcl.
 # We make a sub-widget to list, change, and reorder the symbol pins,
 # similar to the "addliblist" procedure in wrapper.tcl
+#
+# Modified 1/5/06:  If "filename" is non-null, then it points to a SPICE
+# netlist containing a subcircuit.  A symbol is made to correspond to the
+# subcircuit definition, with a link to include the file.
+#
+# Modified 1/7/06:  If "orderedpins" is non-null, then it contains the
+# list of pins in proper order.
 #---------------------------------------------------------------------------
 
-proc xcircuit::makesymbol {} {
+proc xcircuit::makesymbol {{filename ""} {orderedpins ""}} {
    global XCOps
 
    set symbolname [.dialog.textent.txt get]
-   xcircuit::symbol make $symbolname $XCOps(library);
-   set schematicname [schematic get]
+
+   # If "filename" is specified then we have a netlist, not a schematic.
+   # Therefore, create an object but don't use xcircuit::symbol
+
+   if {$filename == ""} {
+      xcircuit::symbol make $symbolname $XCOps(library);
+      set schematicname [schematic get]
+      set noschem 0
+   } else {
+      set schematicname $symbolname
+      set noschem 1
+   }
 
    set pinspace 64
    set halfspace [expr $pinspace / 2]
@@ -70,8 +88,8 @@ proc xcircuit::makesymbol {} {
    set vpins $toppins
    if {$botpins > $toppins} {set vpins $botpins}
 
-   set boxwidth [expr ($hpins + 1) * $pinspace]
-   set boxheight [expr ($vpins + 1) * $pinspace]
+   set boxwidth [expr ($vpins + 1) * $pinspace]
+   set boxheight [expr ($hpins + 1) * $pinspace]
 
    set hwidth [expr $boxwidth / 2]
    if {$hwidth < 256} {set hwidth 256}
@@ -81,6 +99,19 @@ proc xcircuit::makesymbol {} {
    set sbox [polygon make box "-$hwidth -$hheight" "$hwidth $hheight"]
    set pinlabels {}
 
+   # If we didn't make a symbol using xcircuit::symbol, now is the time
+   # to generate the object.
+
+   if {$noschem == 1} {
+      select $sbox
+      set handle [object make $symbolname $XCOps(library)]
+      if {$handle == ""} {
+	 push [object parts]
+      } else {
+         push $handle
+      }
+   }
+
    set x [expr -$hwidth - $qtrspace]
    set y [expr -($leftpins - 1) * $halfspace]
    for {set j 0} {$j < $leftpins} {incr j} {
@@ -166,37 +197,55 @@ proc xcircuit::makesymbol {} {
 
    # Determine if the schematic already has a "subckt" line.  If so,
    # attempt to arrange the pin ordering from it.  If not, create one.
-   set subckttext [xcircuit::getsubckttext $schematicname]
+   if {$noschem == 1} {
+      set subckttext ""
+      set pinlabels $orderedpins
+   } else {
+      set subckttext [xcircuit::getsubckttext $schematicname]
+   }
+
    if {$subckttext == ""} {
       set subckttext [list {Text "spice1:.subckt %n"}]
       foreach j $pinlabels {
          lappend subckttext "$j"
       }
-      schematic goto
-      set bbox [join [page bbox all]] 
-      set x [expr ([lindex $bbox 2] + [lindex $bbox 0]) / 2]
-      set y [expr [lindex $bbox 1] - $pinspace]
-      set nlab [label make info "$subckttext" "$x $y"] 
-      label $nlab justify center
-      set y [expr $y - $pinspace]
-      set nlab [label make info "spice-1:.ends" "$x $y"] 
-      symbol goto
+      if {$noschem == 0} {
+         schematic goto
+         set bbox [join [page bbox all]] 
+         set x [expr ([lindex $bbox 2] + [lindex $bbox 0]) / 2]
+         set y [expr [lindex $bbox 1] - $pinspace]
+         set nlab [label make info "$subckttext" "$x $y"] 
+         label $nlab justify center
+         set y [expr $y - $pinspace]
+         set nlab [label make info "spice-1:.ends" "$x $y"] 
+         symbol goto
+      }
 
       set itext [list {Text "spice:"} {Parameter class} {Parameter idx}]
       set pstring ""
       foreach j $pinlabels {
-         lappend pstring "%p$j"
+	 if {[string length $pstring] > 60} {
+            lappend itext [subst {Text "$pstring"}]
+            lappend itext {Return}
+	    set pstring "+"
+	 }
+	 set pstring [join [list $pstring "%p$j"]]   ;# preserves whitespace
       }
-      lappend pstring "%n"
-      lappend itext [subst {Text " $pstring"}]
+      set pstring [join [list $pstring "%n"]]   ;# preserves whitespace
+      lappend itext [subst {Text "$pstring"}]
    } else {
       set itext [list {Text "spice:"} {Parameter class} {Parameter idx}]
       set pstring ""
       foreach j [lrange [lindex $subckttext 0] 2 end] {
-         lappend pstring "%p$j"
+	 if {[string length $pstring] > 60} {
+            lappend itext [subst {Text "$pstring"}]
+            lappend itext {Return}
+	    set pstring "+"
+	 }
+	 set pstring [join [list $pstring "%p$j"]]   ;# preserves whitespace
       }
-      lappend pstring "%n"
-      lappend itext [subst {Text " $pstring"}]
+      set pstring [join [list $pstring "%n"]]   ;# preserves whitespace
+      lappend itext [subst {Text "$pstring"}]
    }
 
    set y [expr -$hheight - 3 * $pinspace]
@@ -204,9 +253,23 @@ proc xcircuit::makesymbol {} {
    deselect
    set nlab [label make info "$itext" "0 $y"] 
    label $nlab justify center
-   
-   deselect
-   zoom view
+
+   if {$noschem == 1} {
+      deselect
+      if {[string index $filename 0] != "/"} {
+	 set filename [join [concat [pwd] $filename] "/"]
+      }
+      set y [expr -$hheight - 4 * $pinspace]
+      set itext [list {Text "spice@1:%F"}]
+      lappend itext [subst {Text "$filename"}]
+      set nlab [label make info "$itext" "0 $y"]
+      label $nlab justify center
+      deselect
+      pop
+   } else {
+      deselect
+      zoom view
+   }
 }
 
 #---------------------------------------------------------------------------
@@ -277,33 +340,63 @@ proc xcircuit::removeselectedpin {} {
 }
 
 #---------------------------------------------------------------------------
-# Move a pin to the next list
+# Move a pin to the left side of the symbol
 #---------------------------------------------------------------------------
 
-proc xcircuit::moveselectedpin {} {
+proc xcircuit::movepinleft {} {
    set w [xcircuit::getselectedpinwidget]
    if {$w != {}} {
       set idx [$w curselection]
       set pinname [$w get $idx]
       $w delete $idx
-      switch -glob $w {
-	 {.dialog.pinself.left.list} {
-		.dialog.pinself.top.list insert end $pinname
-		.dialog.pinself.top.list selection set end
-	 }
-	 {.dialog.pinself.top.list} {
-		.dialog.pinself.right.list insert end $pinname
-		.dialog.pinself.right.list selection set end
-	 }
-	 {.dialog.pinself.right.list} {
-		.dialog.pinself.bottom.list insert end $pinname
-		.dialog.pinself.bottom.list selection set end
-	 }
-	 {.dialog.pinself.bottom.list} {
-		.dialog.pinself.left.list insert end $pinname
-		.dialog.pinself.left.list selection set end
-	 }
-      }
+      .dialog.pinself.left.list insert end $pinname
+      $w selection set $idx
+   }
+}
+
+#---------------------------------------------------------------------------
+# Move a pin to the top side of the symbol
+#---------------------------------------------------------------------------
+
+proc xcircuit::movepintop {} {
+   set w [xcircuit::getselectedpinwidget]
+   if {$w != {}} {
+      set idx [$w curselection]
+      set pinname [$w get $idx]
+      $w delete $idx
+      .dialog.pinself.top.list insert end $pinname
+      $w selection set $idx
+
+   }
+}
+
+#---------------------------------------------------------------------------
+# Move a pin to the right side of the symbol
+#---------------------------------------------------------------------------
+
+proc xcircuit::movepinright {} {
+   set w [xcircuit::getselectedpinwidget]
+   if {$w != {}} {
+      set idx [$w curselection]
+      set pinname [$w get $idx]
+      $w delete $idx
+      .dialog.pinself.right.list insert end $pinname
+      $w selection set $idx
+   }
+}
+
+#---------------------------------------------------------------------------
+# Move a pin to the bottom side of the symbol
+#---------------------------------------------------------------------------
+
+proc xcircuit::movepinbottom {} {
+   set w [xcircuit::getselectedpinwidget]
+   if {$w != {}} {
+      set idx [$w curselection]
+      set pinname [$w get $idx]
+      $w delete $idx
+      .dialog.pinself.bottom.list insert end $pinname
+      $w selection set $idx
    }
 }
 
@@ -311,7 +404,7 @@ proc xcircuit::moveselectedpin {} {
 # Create the pin arranger widget and add it to the dialog box.
 #---------------------------------------------------------------------------
 
-proc xcircuit::addpinarranger {w} {
+proc xcircuit::addpinarranger {w {pinlist {}}} {
 
    xcircuit::removepinarranger ${w}
 
@@ -332,13 +425,13 @@ proc xcircuit::addpinarranger {w} {
    listbox ${w}.pinself.bottom.list
 
    pack ${w}.pinself.left.title -side top
-   pack ${w}.pinself.left.list -side top
+   pack ${w}.pinself.left.list -side top -fill y -expand true
    pack ${w}.pinself.top.title -side top
-   pack ${w}.pinself.top.list -side top
+   pack ${w}.pinself.top.list -side top -fill y -expand true
    pack ${w}.pinself.right.title -side top
-   pack ${w}.pinself.right.list -side top
+   pack ${w}.pinself.right.list -side top -fill y -expand true
    pack ${w}.pinself.bottom.title -side top
-   pack ${w}.pinself.bottom.list -side top
+   pack ${w}.pinself.bottom.list -side top -fill y -expand true
 
    grid ${w}.pinself.left -row 0 -column 0 -sticky news -padx 1 -pady 1
    grid ${w}.pinself.top -row 0 -column 1 -sticky news -padx 1 -pady 1
@@ -350,7 +443,23 @@ proc xcircuit::addpinarranger {w} {
    grid columnconfigure ${w}.pinself 2 -weight 1 -minsize 50
    grid columnconfigure ${w}.pinself 3 -weight 1 -minsize 50
 
-   set pinlist [xcircuit::getpinlist [page label]]
+   grid rowconfigure ${w}.pinself 0 -weight 1 -minsize 50
+
+   # Determine if the pinlist is fixed by either being taken from a "subckt"
+   # line in a schematic, or being taken from a "subckt" line in a SPICE deck.
+   # If so, we pass the ordered list to the symbol construction routine, and
+   # we also prevent symbol pins from being deleted.  
+
+   if {$pinlist == {}} {
+      set pinlist [xcircuit::getpinlist [page label]]
+      if {[xcircuit::getsubckttext [page label]] != {}} {
+         set orderedpins 1
+      } else {
+         set orderedpins 0
+      }
+   } else {
+      set orderedpins 1
+   }
 
    # Break the pinlist up into 4 parts
 
@@ -377,15 +486,28 @@ proc xcircuit::addpinarranger {w} {
       ${w}.pinself.bottom.list insert end [lindex $pinlist $k]
    }
 
-   pack ${w}.pinself -side top -anchor w -padx 20 -pady 5
+   pack ${w}.pinself -side top -anchor w -padx 20 -pady 5 -fill y -expand true
 
-   button ${w}.bbar.remove -text "Remove Pin" -bg beige -command \
+   if {$orderedpins == 0} {
+      button ${w}.bbar.remove -text "Remove Pin" -bg beige -command \
 	{xcircuit::removeselectedpin}
-   button ${w}.bbar.move -text "Move Pin" -bg beige -command \
-	{xcircuit::moveselectedpin}
+   }
+   button ${w}.bbar.moveleft -text "Move Left" -bg beige -command \
+	{xcircuit::movepinleft}
+   button ${w}.bbar.movetop -text "Move Top" -bg beige -command \
+	{xcircuit::movepintop}
+   button ${w}.bbar.moveright -text "Move Right" -bg beige -command \
+	{xcircuit::movepinright}
+   button ${w}.bbar.movebottom -text "Move Bottom" -bg beige -command \
+	{xcircuit::movepinbottom}
 
-   pack ${w}.bbar.remove -side left -ipadx 10
-   pack ${w}.bbar.move -side left -ipadx 10
+   if {$orderedpins == 0} {
+      pack ${w}.bbar.remove -side left -ipadx 10
+   }
+   pack ${w}.bbar.moveleft -side left -ipadx 10
+   pack ${w}.bbar.movetop -side left -ipadx 10
+   pack ${w}.bbar.moveright -side left -ipadx 10
+   pack ${w}.bbar.movebottom -side left -ipadx 10
 }
 
 #---------------------------------------------------------------------------
@@ -396,6 +518,11 @@ proc xcircuit::removepinarranger {w} {
    catch {
       pack forget ${w}.pinself
       destroy ${w}.pinself
+      pack forget ${w}.bbar.movebottom
+      pack forget ${w}.bbar.moveright
+      pack forget ${w}.bbar.movetop
+      pack forget ${w}.bbar.moveleft
+      pack forget ${w}.bbar.remove
    }
 } 
 
@@ -429,3 +556,69 @@ proc xcircuit::promptmakesymbol {{name ""}} {
   xcircuit::addpinarranger .dialog
 }        
 
+#---------------------------------------------------------------------------
+# Routine which parses a spice file for the first .subckt line and
+# generates a symbol to match, with a "%F" escape pointing to the
+# spice file to include.
+#---------------------------------------------------------------------------
+
+proc xcircuit::spice2symbol {filename {subcktname ""}} {
+  global XCOps
+
+  set f [open $filename]
+  set infolabel ""
+  while {[gets $f line] >= 0} {
+     if {[string first .subckt $line] == 0} {
+	while {[gets $f nextline] >= 0} {
+	   if {[string first + $nextline] != 0} {break}
+	   append line [string range $nextline 1 end]
+	}
+	set infolabel $line
+	if {$subcktname == ""} {break}
+	if {[string compare $subcktname [lindex $infolabel 1]] == 0} {break}
+     }
+  }
+  close $f
+
+  if {[string length $infolabel] == 0} {return}
+  set pinlabels [lrange $infolabel 2 end]
+
+  .dialog.bbar.okay configure -command \
+          "if {[string first {Page } [page label]] >= 0} { \
+          page label [.dialog.textent.txt get]}; \
+          xcircuit::makesymbol $filename [list $pinlabels]; \
+          wm withdraw .dialog"
+  .dialog.textent.title.field configure -text "Confirm symbol name:"
+  .dialog.textent.txt delete 0 end
+  .dialog.textent.txt insert 0 [lindex $infolabel 1]
+  xcircuit::popupdialog
+  xcircuit::addliblist .dialog "Place in: "
+  xcircuit::addpinarranger .dialog $pinlabels
+}
+
+#---------------------------------------------------------------------------
+# Procedure that creates the dialog to find a spice file to parse and
+# calls spice2symbol.
+#---------------------------------------------------------------------------
+
+proc xcircuit::promptspicesymbol {} {
+  global XCOps
+  .filelist.bbar.okay configure -command \
+	{xcircuit::spice2symbol [.filelist.textent.txt get] ; \
+	wm withdraw .filelist}
+  .filelist.listwin.win configure -data "cir"
+  .filelist.textent.title.field configure -text "Select spice file to parse:"
+  .filelist.textent.txt delete 0 end
+  xcircuit::popupfilelist
+  xcircuit::removeliblist .filelist
+}
+
+#---------------------------------------------------------------------------
+# Add a menu item to invoke promptspicesymbol.
+#---------------------------------------------------------------------------
+
+set m .xcircuit.menubar.netlistbutton.netlistmenu
+$m insert 7 command -label "SPICE to symbol" -command {xcircuit::promptspicesymbol}
+unset m
+
+#---------------------------------------------------------------------------
