@@ -46,9 +46,8 @@
 
 extern char _STR2[250], _STR[150];
 extern Globaldata xobjs;
-extern Clientdata areastruct;
+extern XCWindowData *areawin;
 extern Display *dpy;
-extern short eventmode;
 extern int *appcolors;
 extern int number_colors;
 extern colorindex *colorlist;
