@@ -43,7 +43,7 @@
 extern char _STR2[250], _STR[150];
 extern fontinfo *fonts;
 extern short fontcount;
-extern Clientdata areastruct;
+extern XCWindowData *areawin;
 extern Globaldata xobjs;
 extern short beeper;
 extern int *appcolors;
@@ -68,9 +68,9 @@ short execcommand(short pflags, char *cmdptr)
    Boolean a;
    char type[50], value[50];
    char *argptr;
-   short cpage = areastruct.page;
+   short cpage = areawin->page;
 
-   /* Note:  areastruct.page is set to 0 before loading the .xcircuitrc	*/
+   /* Note:  areawin->page is set to 0 before loading the .xcircuitrc	*/
    /*file, so .xcircuitrc file commands act on page 0 only;  from the	*/
    /* command line they act on the current page.			*/
 
@@ -91,24 +91,23 @@ short execcommand(short pflags, char *cmdptr)
    else if (!(a = strcmp(cmdptr, "enable")) || !strcmp(cmdptr, "disable")) {
       sscanf(argptr, "%149s", _STR);
       if (!strncmp(_STR, "grid", 4)) {
-	 areastruct.gridon = a ? True : False;
-	 toggle(GridGridButton, &areastruct.gridon, NULL);
+	 areawin->gridon = a ? True : False;
+	 toggle(GridGridButton, &areawin->gridon, NULL);
       }
       else if (!strncmp(_STR, "snap", 4) || !strncmp(_STR, "snap-to", 4)) {
-	 areastruct.snapto = a ? True : False;
-	 toggle(SnaptoSnaptoButton, &areastruct.snapto, NULL);
+	 areawin->snapto = a ? True : False;
+	 toggle(SnaptoSnaptoButton, &areawin->snapto, NULL);
       }
       else if (!strcmp(_STR, "axis") || !strcmp(_STR, "axes")) {
-	 areastruct.axeson = a ? True : False;
-	 toggle(GridAxesButton, &areastruct.axeson, NULL);
+	 areawin->axeson = a ? True : False;
+	 toggle(GridAxesButton, &areawin->axeson, NULL);
       }
       else if (!strcmp(_STR, "xschema") || !strcmp(_STR, "schema")) {
-	 areastruct.schemon = a ? True : False;
-	 doxschema(OptionsEnableXSchemaButton, 1, NULL);
+	 /* Do nothing---retained for backward compatibility only */
       }
 #ifdef HAVE_XPM
       else if (!strcmp(_STR, "toolbar")) {
-	 areastruct.toolbar_on = a ? True : False;
+	 areawin->toolbar_on = a ? True : False;
 	 dotoolbar(OptionsDisableToolbarButton, NULL, NULL);
       }
 #endif
@@ -143,8 +142,8 @@ short execcommand(short pflags, char *cmdptr)
    /* load extra files */
 
    else if (!strcmp(cmdptr, "load")) {
-      int pageno = areastruct.page;
-      int savepage = areastruct.page;
+      int pageno = areawin->page;
+      int savepage = areawin->page;
       if (sscanf(argptr, "%249s %d", _STR, &pageno) >= 1) {
 	 if (pageno != savepage) newpage(pageno);
 	 startloadfile();
@@ -219,7 +218,10 @@ short execcommand(short pflags, char *cmdptr)
 	    keywstate = -1;
 	    func = string_to_func(_STR, NULL);
 	 }
-	 binding = binding_to_string(keywstate, func);
+	 if (keywstate == -1)
+	    binding = function_binding_to_string(0, func);
+	 else
+	    binding = key_binding_to_string(0, keywstate);
 	 Wprintf("%s", binding);
 	 free(binding);
       }
@@ -267,7 +269,7 @@ short execcommand(short pflags, char *cmdptr)
 
       refresh(NULL, NULL, NULL);
 
-      while(XCheckWindowEvent(dpy, areastruct.areawin, ~NoEventMask, &event))
+      while(XCheckWindowEvent(dpy, areawin->window, ~NoEventMask, &event))
          XtDispatchEvent(&event);
    }
 
@@ -287,7 +289,7 @@ short execcommand(short pflags, char *cmdptr)
       if (!strncmp(argptr, "font", 4)) {
 	 short i;
 	 if (strstr(argptr + 4, "scale")) {
-	    sscanf(argptr + 4, "%*s %f", &areastruct.textscale);
+	    sscanf(argptr + 4, "%*s %f", &areawin->textscale);
 	 }
 	 else {
 	    sscanf(argptr, "%*s %49s", value);
@@ -297,7 +299,7 @@ short execcommand(short pflags, char *cmdptr)
 	    if (i == fontcount) {
 	       loadfontfile(value);
 	    }
-   	    areastruct.psfont = i;
+   	    areawin->psfont = i;
 	    setdefaultfontmarks();
 	 }
       }
@@ -311,14 +313,14 @@ short execcommand(short pflags, char *cmdptr)
       }
       else if (!strncmp(argptr, "line", 4)) {
 	 if (strstr(argptr + 4, "width")) {
-	    sscanf(argptr + 4, "%*s %f", &areastruct.linewidth);
+	    sscanf(argptr + 4, "%*s %f", &areawin->linewidth);
 	 }
       }
       else if (!strncmp(argptr, "colorscheme", 11)) {
 	 sscanf(argptr, "%*s %49s", value);
 	 if (!strcmp(value, "inverse") || !strcmp(value, "2"))
-	    areastruct.invert = False;
-	 inversecolor(NULL, &areastruct.invert, NULL);
+	    areawin->invert = False;
+	 inversecolor(NULL, &areawin->invert, NULL);
       }
       else if (!strncmp(argptr, "coordstyle", 10)) {
 	 sscanf(argptr, "%*s %49s", value);
@@ -464,7 +466,7 @@ void loadrcfile()
    if (!(flags & FONTOVERRIDE)) {
       loadfontfile("Helvetica");
       for (i = 0; i < fontcount; i++)
-	 if (!strcmp(fonts[i].psname, "Helvetica")) areastruct.psfont = i;
+	 if (!strcmp(fonts[i].psname, "Helvetica")) areawin->psfont = i;
    }
    setdefaultfontmarks();
 
