@@ -77,6 +77,26 @@ typedef struct {
 extern Globaldata xobjs;
 extern XCWindowData *areawin;
 
+/*----------------------------------------------------------------------*/
+/* Attempt to set the window.  If the window exists, set it as current	*/
+/* and return TRUE.  Otherwise, return FALSE.  This prevents the undo	*/
+/* mechanism from crashing if we close a window and then try to undo	*/
+/* events that were created relative to it.				*/
+/*----------------------------------------------------------------------*/
+
+Boolean setwindow(XCWindowData *trywindow)
+{
+   XCWindowData *chkwin;
+
+   for (chkwin = xobjs.windowlist; chkwin != NULL; chkwin = chkwin->next) {
+      if (chkwin == trywindow) {
+	 areawin = trywindow;
+	 return TRUE;
+      }
+   }
+   return FALSE;
+}
+
 /*----------------------------------------------------------------------*/
 /* remember_selection ---						*/
 /*									*/
@@ -356,6 +376,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
    newrecord->last = NULL;
    newrecord->type = type;
    newrecord->thisinst =  thisinst;
+   newrecord->window = areawin;
    newrecord->undodata = (char *)NULL;
    newrecord->idata = 0;
 
@@ -564,6 +585,7 @@ short undo_one_action()
    arcptr thisarc;
    splineptr thisspline;
    graphicptr thisgraphic;
+   XCWindowData *savewindow = areawin;
 
    /* Undo the recorded action and shift the undo record pointer.	*/
 
@@ -576,6 +598,13 @@ short undo_one_action()
    xobjs.undostack = thisrecord->next;
    xobjs.redostack = thisrecord;
 
+   /* Set window, if event occurred in a different window */
+   if (setwindow(thisrecord->window) == FALSE) {
+      Wprintf("Error:  Undo event in nonexistant window!  Flushing stack.\n");
+      flush_undo_stack();
+      return 0;
+   }
+
    /* Setting eventmode to UNDO_MODE prevents register_for_undo() from 	*/
    /* being called again while executing the event.			*/
 
@@ -854,6 +883,7 @@ short undo_one_action()
       Fprintf(stderr, "Warning:  Unfinished undo series in stack!\n");
       thisrecord->idx = -thisrecord->idx;
    }
+   areawin = savewindow;
    return thisrecord->idx;
 }
 
@@ -893,6 +923,7 @@ short redo_one_action()
    polyptr thispoly;
    graphicptr thisgraphic;
    objinstptr thisinst;
+   XCWindowData *savewindow = areawin;
 
    /* Undo the recorded action and shift the undo record pointer.	*/
 
@@ -904,6 +935,13 @@ short redo_one_action()
    xobjs.undostack = thisrecord;
    xobjs.redostack = thisrecord->last;
 
+   /* Set window, if event occurred in a different window */
+   if (setwindow(thisrecord->window) == FALSE) {
+      Wprintf("Error:  Undo event in nonexistant window!  Flushing stack.\n");
+      flush_undo_stack();
+      return 0;
+   }
+
    eventmode = UNDO_MODE;
 
    /* type-dependent part */
@@ -1143,6 +1181,8 @@ short redo_one_action()
    }
 
    eventmode = NORMAL_MODE;	/* Set on a per-event-type basis? */
+   areawin = savewindow;
+
    return thisrecord->idx;
 }
 
