@@ -1277,7 +1277,11 @@ typedef struct {
 /* all the critical data needed by the drawing window 			*/
 /*----------------------------------------------------------------------*/
 
-typedef struct {
+typedef struct _windowdata *XCWindowDataPtr;
+
+typedef struct _windowdata {
+
+   XCWindowDataPtr next;	/* next window in list */
 
    /* widgets and X11 parameters */
    xcWidget	area;
@@ -1334,7 +1338,7 @@ typedef struct {
    short	editpart, editsubpart;
    char		*lastbackground;
    Cursor	*defaultcursor;
-} XCWindowData, *XCWindowDataPtr;
+} XCWindowData;
 
 /* Record for undo function */ 
 
@@ -1346,6 +1350,7 @@ typedef struct _undostack {
    u_int type;		/* type of event */
    short idx;		/* counter for undo event */
    objinstptr thisinst;	/* instance of object in which event occurred */
+   XCWindowData *window;/* window in which event occurred */
    int idata;		/* simple undedicated integer datum */
    char *undodata;	/* free space to be malloc'd */
 			/* (size dependent on "type") */
@@ -1389,8 +1394,7 @@ typedef struct {
    objinstptr   *libtop;
    Imagedata	*imagelist;
    short	images;
-   XCWindowData *windowlist;
-   short	windows;
+   XCWindowData *windowlist;	/* linked list of known windows */
 } Globaldata;
 
 /*----------------------------------------------------------------------*/
